
[![Build Status](https://travis-ci.org/pvasa/cameraview-ex.svg?branch=master)](https://travis-ci.org/pvasa/cameraview-ex)
[![Bintray](https://img.shields.io/bintray/v/ryan652/android/cameraview-ex.svg)](https://bintray.com/ryan652/android/cameraview-ex/_latestVersion)
[![License](https://img.shields.io/github/license/pvasa/cameraview-ex.svg)](LICENSE)

# CameraViewEx

_This is an extended version of [Google's cameraview library](https://github.com/google/cameraview)_

*This is a preview release. The API is subject to change.*

CameraViewEx makes integration of camera implementation and various camera features into any Android project very easy.

Requires API Level 14. The library uses Camera 1 API on API Level 14-20 and Camera2 on 21 and above.

| API Level | Camera API | Preview View |
|:---------:|------------|--------------|
| 14-20     | Camera1    | TextureView  |
| 21+       | Camera2    | TextureView  |

## Features

- Camera preview by placing it in a layout XML (and calling the start method)
- Configuration by attributes
  - Camera mode (app:cameraMode) defaults to "single_capture"
  - Facing (app:facing) defaults to "back"
  - Aspect ratio (app:aspectRatio) defaults to "4:3"
  - Auto-focus (app:autoFocus) defaults to "false"
  - Flash (app:flash) defaults to "off"
  - Auto white balance (app:awb) defaults to "off"
  - Optical Stabilization (app:opticalStabilization) defaults to "false"
  - Noise Reduction (app:noiseReduction) defaults to "off"
  - Camera shutter view (app:shutter) defaults to "off"
  - Output format (app:outputFormat) defaults to "jpeg"
  - Zero shutter lag (app:zsl) defaults to "false"

_**Note:** Devices that run **Camera1** implementation will only support **app:aspectRatio**, **app:autoFocus**, and **app:flash** attributes. All others will be ignored. Camera2 implementations (ie. API 21 and above) will support all features._

## Usage

#### Import dependency
In app build.gradle,
```gradle
dependencies {
    // ...
    implementation "com.priyankvasa.android:cameraview-ex:2.2.0"
}
```

#### In layout xml
```xml
<com.priyankvasa.android.cameraviewex.CameraView
    android:id="@+id/camera"
    android:layout_width="match_parent"
    android:layout_height="wrap_content"
    android:adjustViewBounds="true"
    android:keepScreenOn="true"
    app:aspectRatio="4:3"
    app:autoFocus="true"
    app:awb="auto"
    app:cameraMode="single_capture"
    app:facing="back"
    app:flash="auto"
    app:noiseReduction="high_quality"
    app:opticalStabilization="true"
    app:outputFormat="jpeg"
    app:shutter="short_time"
    app:zsl="true" />

<!-- Or to apply all those params as mentioned above use Widget.CameraView style -->

<com.priyankvasa.android.cameraviewex.CameraView
    android:id="@+id/camera"
    android:layout_width="match_parent"
    android:layout_height="wrap_content"
    style="@style/Widget.CameraView" />
```

#### In Fragment
```kotlin
override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
    super.onViewCreated(view, savedInstanceState)
    
    // For camera mode Modes.CameraMode.SINGLE_CAPTURE
    camera.addCameraOpenedListener { Timber.i("Camera opened.") }
        // Output format is whatever set for [app:outputFormat] parameter
        .addPictureTakenListener { imageData: ByteArray -> Timber.i("Picture taken.") }
        .addCameraClosedListener { Timber.i("Camera closed.") }
    
    /* --- OR --- */
    
    // For camera mode Modes.CameraMode.CONTINUOUS_FRAME
    camera.addCameraOpenedListener { Timber.i("Camera opened.") }
        // Output format is always ImageFormat.YUV_420_888
        .setPreviewFrameListener { image: Image -> Timber.i("Preview frame available.") }
        .addCameraClosedListener { Timber.i("Camera closed.") }
}

override fun onResume() {
    super.onResume()
    camera.run { if (!isCameraOpened) start() }
}

override fun onPause() {
    camera.run { if (isCameraOpened) stop() }
    super.onPause()
}

override fun onDestroyView() {
    camera.run { if (isCameraOpened) stop(removeAllListeners = true) }
    super.onDestroyView()
}
```

You can see a complete usage in the [sampleApp](https://github.com/pvasa/cameraview-ex/tree/development/sampleApp) app module.
